#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <assert.h>

#include "peekpoke.h"
#include "ep93xx_adc.h"
#define DATA_PAGE 0x80840000

static double get_volts(int val, int zero, int range);

/* Port C info */
#define PCDR         0x08          //Port C Data Register
#define PCDDR        0x18          //Port C Data Direction Register
#define PC_MASK      0x01          //Port C Mask

void init(unsigned dr_page,unsigned adc_page,unsigned syscon_page) {
  //assert Port C0
  POKE32(dr_page + PCDDR,(PEEK32(dr_page + PCDDR) | PC_MASK));
  POKE32(dr_page + PCDR,(PEEK32(dr_page + PCDR) | PC_MASK));
  //initialize the ep93xx ADC
  init_ADC(adc_page, syscon_page);
}

int EP93XX_ADC_microvolts(unsigned long adc_page) {
	long long r = EP93XX_ADC_read(adc_page);
	r = (((r + 25000) * 100000LL) * 3300000LL) / 5000000000LL;
	return r;	
}

int ADCpoll_main(int argc,char *argv[]) {
  int devmem;
  unsigned adc_page,syscon_page,dr_page;
  int port=0,num=0;

  if (argc < 3) {
    printf("Usage: ADCpoll port# number_of_samples\n");
    printf("\tport# is 0 through 3, or -1 for all ports.\n");
    return 1;
  }
  port = atoi(argv[1]);
  num = atoi(argv[2]);
  if (port < -1 || port > 3) {
    printf("Port must be between 0 and 3.\n");
    printf("Use port -1 to sample all 4 ports.\n");
    return 1;
  }

  devmem = open("/dev/mem", O_RDWR|O_SYNC);
  assert(devmem != -1);
  dr_page = (unsigned long)mmap(0, getpagesize(), PROT_READ|PROT_WRITE
				, MAP_SHARED, devmem, DATA_PAGE);
  assert(&dr_page != MAP_FAILED);
  adc_page = (unsigned long)mmap(0, getpagesize(), PROT_READ|PROT_WRITE,
				 MAP_SHARED, devmem, ADC_PAGE);
  assert(&adc_page != MAP_FAILED);
  syscon_page = (unsigned long)mmap(0, getpagesize(), PROT_READ|PROT_WRITE
				    , MAP_SHARED, devmem, SYSCON_PAGE);
  assert(&syscon_page != MAP_FAILED);

  init(dr_page,adc_page,syscon_page);

  if (port >= 0) {
    EP93XX_ADC_select(adc_page,port);
  }
  while (num--) {
    if (port >= 0) {
      printf("%d\n",EP93XX_ADC_microvolts(adc_page));
    } else {
      EP93XX_ADC_select(adc_page,0);
      printf("%d ",EP93XX_ADC_microvolts(adc_page));
      EP93XX_ADC_select(adc_page,1);
      printf("%d ",EP93XX_ADC_microvolts(adc_page));
      EP93XX_ADC_select(adc_page,2);
      printf("%d ",EP93XX_ADC_microvolts(adc_page));
      EP93XX_ADC_select(adc_page,3);
      printf("%d\n",EP93XX_ADC_microvolts(adc_page));
    }
  }
  close(devmem);
  return 0;
}
