#define ADC_PAGE          0x80900000
#define ADCRESULT_OFFSET  0x0008
#define SDR_MASK          0x80000000
#define DATA_OFFSET       0x0008
#define DATA_MASK         0xFFFF
#define ADCSWITCH_OFFSET  0x0018
#define ADC_CH0           0x0608
#define ADC_CH1           0x0680
#define ADC_CH2           0x0640
#define ADC_CH3           0x0620
#define ADC_CH4           0x0610
#define ADCSWLOCK_OFFSET  0x0020
#define UNLOCK_VAL        0xAA

#define SYSCON_PAGE       0x80930000
#define ADCCLKDIV_OFFSET  0x0090
#define SYSCON_UNLOCK     0x00C0
#define TSEN_MASK         0x80000000
#define DEVICECFG_OFFSET  0x0080
#define ADCPD_MASK        0x02
#define ADCEN_MASK        0x20000
#define FALSE 0
#define TRUE !FALSE

/*  prototypes  */
void init_ADC(unsigned long adc_page, unsigned long syscon_page);
static char is_ADC_busy(unsigned long adc_page);

void init_ADC(unsigned long adc_page, unsigned long syscon_page)
{
	unsigned long val;

	/*    set TSEN bit     */
	val = PEEK32(syscon_page + ADCCLKDIV_OFFSET);
	//unlock the software lock
	POKE32(syscon_page + SYSCON_UNLOCK, UNLOCK_VAL);
	POKE32(syscon_page + ADCCLKDIV_OFFSET, TSEN_MASK | val);

	/*    set ADCEN bit    */
	val = PEEK32(syscon_page + DEVICECFG_OFFSET);
	POKE32(adc_page + SYSCON_UNLOCK, UNLOCK_VAL); //unlock the soft lock
	POKE32(syscon_page + DEVICECFG_OFFSET, val | ADCEN_MASK);

	/*    clear ADCPD bit  */
	val = PEEK32(syscon_page + DEVICECFG_OFFSET);
	POKE32(adc_page + SYSCON_UNLOCK, UNLOCK_VAL); //unlock the soft lock
	POKE32(syscon_page + DEVICECFG_OFFSET, val & ~ADCPD_MASK);
}

void EP93XX_ADC_select(unsigned long adc_page,unsigned short channel) {
  unsigned short cur_ch;

  switch(channel) {
  default:
  case 0:
    cur_ch = ADC_CH0;
    break;
  case 1:
    cur_ch = ADC_CH1;
    break;
  case 2:
    cur_ch = ADC_CH2;
    break;
  case 3:
    cur_ch = ADC_CH3;
    break;
  case 4:
    cur_ch = ADC_CH4;
    break;
  }

  POKE32(adc_page + ADCSWLOCK_OFFSET, UNLOCK_VAL); //unlock the soft lock

  //write ADCSwitch reg to select channel
  POKE32(adc_page + ADCSWITCH_OFFSET, cur_ch);

  // Now wait for the channel to switch
  // p.34 of the data sheet (which one? all i have is one page printout)
  // says the channel switch settling time is 500us or 2ms depending on
  // the value of ADIV, which relates the input clock to the ADC module
  // to the 14.7456Mhz clock.
  // For now, we will just call usleep for 2ms, which will probably return
  // after one clock tick.
  usleep(2000);
}

short EP93XX_ADC_read(unsigned long adc_page) {
  unsigned long val;

  // loop until bit 31 clear
  do {
    val = PEEK32(adc_page + DATA_OFFSET) ;
  } while (val & 0x80000000);

  // loop until bit 31 set
  do {
    val = PEEK32(adc_page + DATA_OFFSET) ;
  } while (!(val & 0x80000000));

  val = val & DATA_MASK;
  usleep(2000);
  
  return (short)val;	
}
